﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace MediaElementDemo
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Tombstoning helpers
        int radioButtonIndex = -1;
        MediaElementState mediaStartState = MediaElementState.Playing;
        TimeSpan mediaStartPosition = TimeSpan.Zero;

        // To prevent Slider reentry
        bool settingSliderFromPlay;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            // Set fields of appbar buttons; otherwise they're null
            rewindButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            playButton = this.ApplicationBar.Buttons[1] as ApplicationBarIconButton;
            pauseButton = this.ApplicationBar.Buttons[2] as ApplicationBarIconButton;
            endButton = this.ApplicationBar.Buttons[3] as ApplicationBarIconButton;

            // Check if file is in isolated storage; otherwise start downloading it
            using (IsolatedStorageFile isoStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isoStore.FileExists(isoStoreRadioButton.Tag as string))
                {
                    isoStoreRadioButton.IsEnabled = true;
                }
                else
                {
                    WebClient webClient = new WebClient();
                    webClient.OpenReadCompleted += OnWebClientOpenReadCompleted;
                    webClient.OpenReadAsync(new Uri("http://www.archive.org/download/BrahmsViolinConcerto-Heifetz/03Iii.AllegroGiocosoMaNonTroppoVivace.mp3"));
                }
            }

            // Update the elapsed time and slider position
            CompositionTarget.Rendering += (sender, args) =>
            {
                TimeSpan pos = mediaElement.Position;
                TimeSpan dur = mediaElement.NaturalDuration.TimeSpan;

                elapsedTime.Text = pos.ToString("m\\:ss\\.f");
                totalTime.Text = dur.ToString("m\\:ss\\.f");
                mediaState.Text = mediaElement.CurrentState.ToString();

                settingSliderFromPlay = true;

                if (dur.Ticks > 0)
                    slider.Value = (double)pos.Ticks / dur.Ticks;
                else
                    slider.Value = 0;

                settingSliderFromPlay = false;
            };
        }

        // When the music file is downloaded, save it to isolated storage
        void OnWebClientOpenReadCompleted(object sender, OpenReadCompletedEventArgs args)
        {
            if (!args.Cancelled && args.Error == null)
            {
                Stream inpStream = args.Result;
                byte[] buffer = new byte[inpStream.Length];
                inpStream.Read(buffer, 0, buffer.Length);

                using (IsolatedStorageFile isoStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    string isoPathName = isoStoreRadioButton.Tag as string;
                    string isoDirName = Path.GetDirectoryName(isoPathName);

                    if (!isoStore.DirectoryExists(isoDirName))
                    {
                        isoStore.CreateDirectory(isoDirName);
                    }

                    using (IsolatedStorageFileStream isoStream = isoStore.CreateFile(isoPathName))
                    {
                        isoStream.Write(buffer, 0, buffer.Length);
                        isoStoreRadioButton.IsEnabled = true;
                    }
                }
            }
        }

        void OnRadioButtonChecked(object sender, RoutedEventArgs args)
        {
            RadioButton radioButton = sender as RadioButton;
            string uriString = radioButton.Tag as string;

            // Save index for tombstoning
            radioButtonIndex = radioButtonPanel.Children.IndexOf(radioButton);

            if (radioButton == isoStoreRadioButton)
            {
                // Call SetSource on MediaElement using isolated-storage stream.
                using (IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    using (Stream isoStream = storage.OpenFile(uriString, FileMode.Open))
                    {
                        mediaElement.SetSource(isoStream);
                    }
                }
            }
            else
            {
                // Set Source property on MediaElement using URI
                Uri uri = new Uri(uriString, uriString.Contains(':') ? UriKind.Absolute : UriKind.Relative);
                mediaElement.Source = uri;
            }
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            if (radioButtonIndex != -1)
            {
                this.State.Add("radioButtonIndex", radioButtonIndex);
                this.State.Add("mediaElementCurrentState", mediaElement.CurrentState);
                this.State.Add("mediaElementPosition", mediaElement.Position);

                // Uncheck checked RadioButton
                (radioButtonPanel.Children[radioButtonIndex] as RadioButton).IsChecked = false;
                radioButtonIndex = -1;
            }
            base.OnNavigatedFrom(args);
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            if (this.State.ContainsKey("radioButtonIndex"))
            {
                // Get the entries
                int radioButtonIndex = (int)this.State["radioButtonIndex"];
                mediaStartState = (MediaElementState)this.State["mediaElementCurrentState"];
                mediaStartPosition = (TimeSpan)this.State["mediaElementPosition"];

                // Remove the entries
                this.State.Remove("radioButtonIndex");
                this.State.Remove("mediaElementPosition");
                this.State.Remove("mediaElementCurrentState");

                // Start the ball rolling
                (radioButtonPanel.Children[radioButtonIndex] as RadioButton).IsChecked = true;
            }

            base.OnNavigatedTo(args);
        }

        private void OnMediaElementMediaOpened(object sender, RoutedEventArgs e)
        {
            switch (mediaStartState)
            {
                case MediaElementState.Playing: mediaElement.Play(); break;
                case MediaElementState.Paused: mediaElement.Pause(); break;
            }
            mediaElement.Position = mediaStartPosition;

            // Set fields to default values
            mediaStartState = MediaElementState.Playing;
            mediaStartPosition = TimeSpan.Zero;
        }

        void OnMediaElementMediaFailed(object sender, ExceptionRoutedEventArgs args)
        {
            errorTextBlock.Text = args.ErrorException.Message;

            if (radioButtonIndex != -1)
            {
                // Uncheck checked RadioButton
                (radioButtonPanel.Children[radioButtonIndex] as RadioButton).IsChecked = false;
                radioButtonIndex = -1;
            }
        }

        void OnMediaElementCurrentStateChanged(object sender, RoutedEventArgs e)
        {
            switch (mediaElement.CurrentState)
            {
                case MediaElementState.Playing:
                    rewindButton.IsEnabled = true;
                    playButton.IsEnabled = false;
                    pauseButton.IsEnabled = true;
                    endButton.IsEnabled = true;
                    break;

                case MediaElementState.Paused:
                case MediaElementState.Stopped:
                    rewindButton.IsEnabled = true;
                    playButton.IsEnabled = true;
                    pauseButton.IsEnabled = false;
                    endButton.IsEnabled = true;
                    break;

                default:
                    rewindButton.IsEnabled = false;
                    playButton.IsEnabled = false;
                    pauseButton.IsEnabled = false;
                    endButton.IsEnabled = false;
                    break;
            }
        }

        void OnSliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            if (!settingSliderFromPlay)
            {
                long ticks = (long)(args.NewValue * mediaElement.NaturalDuration.TimeSpan.Ticks);
                mediaElement.Position = new TimeSpan(ticks);
            }
        }

        void OnRewindButtonClick(object sender, EventArgs args)
        {
            mediaElement.Position = TimeSpan.Zero;
        }

        void OnPlayButtonClick(object sender, EventArgs args)
        {
            mediaElement.Play();
        }

        void OnPauseButtonClick(object sender, EventArgs args)
        {
            mediaElement.Pause();
        }

        void OnEndButtonClick(object sender, EventArgs args)
        {
            mediaElement.Position = mediaElement.NaturalDuration.TimeSpan;
        }
    }
}